==> xchange-stream-gemini-v2-5.1.2.4-sources.jar: info/bitrich/xchangestream/gemini/GeminiStreamingExchange.java <==
package info.bitrich.xchangestream.gemini;

import info.bitrich.xchangestream.core.ProductSubscription;
import info.bitrich.xchangestream.core.StreamingExchange;
import info.bitrich.xchangestream.core.StreamingMarketDataService;
import info.bitrich.xchangestream.service.netty.ConnectionStateModel.State;
import io.reactivex.Completable;
import io.reactivex.Observable;
import org.knowm.xchange.ExchangeSpecification;
import org.knowm.xchange.exceptions.NotYetImplementedForExchangeException;
import org.knowm.xchange.gemini.v1.GeminiExchange;

/** Adapted from V1 by Max Gao on 01-09-2021 */
public class GeminiStreamingExchange extends GeminiExchange implements StreamingExchange {
  private static final String API_V2_URI = "wss://api.gemini.com/v2/marketdata/";

  private GeminiStreamingService streamingService;
  private GeminiStreamingMarketDataService streamingMarketDataService;

  public GeminiStreamingExchange() {}

  @Override
  protected void initServices() {
    super.initServices();
  }

  @Override
  public Completable connect(ProductSubscription... args) {
    if (args == null || args.length == 0)
      throw new UnsupportedOperationException("The ProductSubscription must be defined!");
    ExchangeSpecification exchangeSpec = getExchangeSpecification();

    this.streamingService = new GeminiStreamingService(API_V2_URI);
    applyStreamingSpecification(exchangeSpec, this.streamingService);

    this.streamingMarketDataService = new GeminiStreamingMarketDataService(streamingService);
    streamingService.subscribeMultipleCurrencyPairs(args);
    return streamingService.connect();
  }

  @Override
  public ExchangeSpecification getDefaultExchangeSpecification() {
    ExchangeSpecification spec = super.getDefaultExchangeSpecification();
    spec.setShouldLoadRemoteMetaData(false);

    return spec;
  }

  @Override
  public Completable disconnect() {
    GeminiStreamingService service = streamingService;
    streamingService = null;
    streamingMarketDataService = null;
    return service.disconnect();
  }

  @Override
  public StreamingMarketDataService getStreamingMarketDataService() {
    return streamingMarketDataService;
  }

  @Override
  public Observable<State> connectionStateObservable() {
    return streamingService.subscribeConnectionState();
  }

  @Override
  public boolean isAlive() {
    return streamingService != null && streamingService.isSocketOpen();
  }

  @Override
  public void useCompressedMessages(boolean compressedMessages) {
    throw new NotYetImplementedForExchangeException();
  }
}

==> xchange-stream-gemini-5.0.4-sources.jar: info/bitrich/xchangestream/gemini/GeminiStreamingExchange.java <==
package info.bitrich.xchangestream.gemini;

import info.bitrich.xchangestream.core.ProductSubscription;
import info.bitrich.xchangestream.core.StreamingExchange;
import info.bitrich.xchangestream.core.StreamingMarketDataService;
import info.bitrich.xchangestream.service.netty.ConnectionStateModel.State;
import io.reactivex.Completable;
import io.reactivex.CompletableEmitter;
import io.reactivex.Observable;
import org.knowm.xchange.exceptions.NotYetImplementedForExchangeException;
import org.knowm.xchange.gemini.v1.GeminiExchange;

/** Created by Lukas Zaoralek on 15.11.17. */
public class GeminiStreamingExchange extends GeminiExchange implements StreamingExchange {
  private static final String API_BASE_URI = "wss://api.gemini.com/v1/marketdata/";

  private final GeminiStreamingService streamingService;
  private GeminiStreamingMarketDataService streamingMarketDataService;

  public GeminiStreamingExchange() {
    this.streamingService = new GeminiStreamingService(API_BASE_URI);
  }

  @Override
  protected void initServices() {
    super.initServices();
    streamingMarketDataService = new GeminiStreamingMarketDataService(streamingService);
  }

  @Override
  public Completable connect(ProductSubscription... args) {
    return Completable.create(CompletableEmitter::onComplete);
  }

  @Override
  public Completable disconnect() {
    return Completable.create(CompletableEmitter::onComplete);
  }

  @Override
  public StreamingMarketDataService getStreamingMarketDataService() {
    return streamingMarketDataService;
  }

  @Override
  public Observable<State> connectionStateObservable() {
    return streamingService.connectionStateObservable();
  }

  @Override
  public boolean isAlive() {
    return streamingService.isAlive();
  }

  @Override
  public void useCompressedMessages(boolean compressedMessages) {
    throw new NotYetImplementedForExchangeException();
  }
}
